function STDP_network_simulation(type, duration, simnum)

% duration should be in seconds (full simulation time is on the order of 60 seconds)

% type is a [n x m]  matrix of 1 and 0's where 1 represents cSTDP and 0
% represented rSTDP. Is ecodes the connectivity of the layers.
% type_ij is the type of STDP from layer j to layer i
% computational layer 1 --> biological layer 4
% computational layer 2 --> biological layer 2+3
% computational layer 3 --> biological layer 5+6
% example: type=[1,1,1;1,1,0;0,1,1]
% or I can input type as a vector eg type = [1,1,1,1,1,0,0,1,1] and reshape
% it in the code

% simnum is which number simulation of this type is being run

% Author: Joe Olson
% November January 31, 2018

% Simulation based off of Kozloski and Cecchi's "A theory of loop formation
% and elimination by spike timing-dependent plasticity". Their
% corresponding code is named STDP_network.

%% Other parameters that are sometimes changed
num_neuro = 99;
NUMEXCSRC1 = 2500;
NUMEXCSRC23 = 2500;
PRATE1 = 20;
PRATE23 = 20;
NUMINHSRC = 1250;
exc1=350;
exc2=275;
exc3=275;
inh1 = 250;
inh2 = 250;
inh3 = 250;
Aratio=1;
MU=0.1;

%% Cell Parameters

% numlay = 2;                                                 % number of layers
numlay1 = num_neuro/3;                                              % Number of neurons in layer 1
numlay2 = num_neuro/3;                                              % number of neurons in layer 2
numlay3 = num_neuro/3;
numexcsrc1 = NUMEXCSRC1;                                            % number of neurons in the thalamus
numexcsrc23 = NUMEXCSRC23;                                          % number of excitatory inputs from higher layers
numinhsrc = NUMINHSRC;                                               % number of inhibitory (GABAergic) neurons
numneuro = numlay1 + numlay2 + numlay3;
layermap = [ones(1,numlay1)*1,ones(1,numlay2)*2,ones(1,numlay3)*3];    % Number coresponds to the layer
type = reshape(type,[3,3])';

Ts = duration;                                     % simulation total time in seconds
T = Ts*1000;                                          % simulation total time in ms
dt = 0.1;                                         % integration time-step in ms
k = T/dt;                                           % total number of iterations

% layer 1
numexccon1 = exc1; numinhcon1 = inh1;
% layer 2
numexccon2 = exc2; numinhcon2 = inh2;
% layer 3
numexccon3 = exc3; numinhcon3 = inh3;

gexcintra = zeros(numneuro,numneuro,'single');                              % initial conditions are zero
gexcintra = gexcintra - diag(diag(gexcintra));          % no self loops
gexcextra1 = zeros(numlay1,numexccon1,'single');           % initial conditions are zero
gexcextra2 = zeros(numlay2,numexccon2,'single');           % initial conditions are zero
gexcextra3 = zeros(numlay3,numexccon3,'single');           % initial conditions are zero
ginh1 = zeros(numlay1,numinhcon1,'single');                            % initial conditions are zero
ginh2 = zeros(numlay2,numinhcon2,'single');                            % initial conditions are zero
ginh3 = zeros(numlay3,numinhcon3,'single');                            % initial conditions are zero

memsave = 100;                                                       % will save dgexcextra only this many timesteps in order to save memory
memsave2 =100;

dgexcmin = 0;                                             % minimum values dgexcintra and dgexcextra can take
dgexcmax = 0.01;                                         % maximum values dgexcintra and dgexcextra can take
dginh = 0.015;                               % delta inhibatory conductance in uS. 0.015 is initial conditions

%using convention w(i,j) is weight from neuron j to neuron i
weightsextra1 = zeros(numlay1,numexccon1,k/memsave2,'single');
weightsextra2 = zeros(numlay2,numexccon2,k/memsave2,'single');
weightsextra3 = zeros(numlay3,numexccon3,k/memsave2,'single');
dgexcintra = (dgexcmax/2).*ones(numneuro,numneuro,'single');                          % delta excitatory conductance in uS for intranetwork synapses. These are considered the actual synaptic weights! 0.005 is initial conditions
dgexcintra = dgexcintra - diag(diag(dgexcintra));
dgexcextra1 = dgexcmax.*ones(numlay1,numexccon1,'single');                  % delta excitatory conductance in uS for extranetwork synapses. 0.01 is initial conditions
dgexcextra2 = dgexcmax.*ones(numlay2,numexccon2,'single');                  % delta excitatory conductance in uS for extranetwork synapses. 0.01 is initial conditions
dgexcextra3 = dgexcmax.*ones(numlay3,numexccon3,'single'); 

% making probablistic connections if more than 100 leaky I&F neurons in our
% network
interprob = 100/numneuro;                            % probability that neurons within a layer are connected
intraprob = 100/numneuro;                           % probability that neurons across layers are connected
if interprob > 1
    interprob = 1;
end
if intraprob >1
    intraprob = 1;
end
dum1 = rand(numlay1,numlay1); dum1(dum1<interprob) = 1; dum1(dum1<1) = 0;
dum2 = rand(numlay1,numlay2); dum2(dum2<intraprob) = 1; dum2(dum2<1) = 0;
dum3 = rand(numlay1,numlay3); dum3(dum3<intraprob) = 1; dum3(dum3<1) = 0;
dum4 = rand(numlay2,numlay1); dum4(dum4<intraprob) = 1; dum4(dum4<1) = 0;
dum5 = rand(numlay2,numlay2); dum5(dum5<interprob) = 1; dum5(dum5<1) = 0;
dum6 = rand(numlay2,numlay3); dum6(dum6<intraprob) = 1; dum6(dum6<1) = 0;
dum7 = rand(numlay3,numlay1); dum7(dum7<intraprob) = 1; dum7(dum7<1) = 0;
dum8 = rand(numlay3,numlay2); dum8(dum8<intraprob) = 1; dum8(dum8<1) = 0;
dum9 = rand(numlay3,numlay3); dum9(dum9<interprob) = 1; dum9(dum9<1) = 0;
% sparse connection matrix
spse = [dum1,dum2,dum3; dum4,dum5,dum6; dum7,dum8,dum9]; spse = spse - diag(diag(spse)); clear dum1 dum2 dum3 dum4
dgexcintra = dgexcintra.*spse;
weights = zeros(numneuro,numneuro,k/memsave,'single'); weights(:,:,1) = dgexcintra.*100;

% STDP parameters
Ap = 0.00035;
An = Ap*Aratio;
mu = MU;
taup = 20;                             % time constant in ms
taun = 20;                             % time constant in ms
Pextra1 = ones(numlay1,numexccon1,'single').*Ap;       % parameter governing synaptic potentiation of the 100 intranetwork neurons from the extranetwork neurons
Pextra2 = ones(numlay2,numexccon2,'single').*Ap;       % parameter governing synaptic potentiation of the 100 intranetwork neurons from the extranetwork neurons
Pextra3 = ones(numlay3,numexccon3,'single').*Ap;
Pintra = ones(1,numneuro,'single').*Ap;        % parameter governing synaptic potentiation of the 100 intranetwork neurons from the intranetwork neurons
M = ones(numneuro,1,'single').*(-An);                  % parameter governing synaptic depression of the 100 intranetwork neurons

% neuron parameters
taum = 20;                          % membrane time constant in ms
tauexc = 5;                         % excitatory conductance time const in ms
tauinh = 5;                       % inhibitory conductance time const in ms
Vrest = -60;                          % rest potential in mV
Eexc = 0;                           % excitatory back potential in mV
Einh = -70;                           % inhibitory back potential in mV
Vthresh = -54;                      % threshold potential in mV
Vreset = -60;                          % reset potential in mV
V = zeros(numneuro,k,'single');                                 % Voltage vector in mV
V(:,1) = Vrest;                                         % intial conditions
spikes = zeros(numneuro,k,'single');                            % will record the spikes emitted from each of the 100 internetwork neurons
gamma = zeros(1,k,'single');

% Fast local inhibitiion parameters
taur = 2;                           % time constant in ms
rmin = 5;                           % minimum rate of poisson inhibition train in Hz
rmax = 1000;                    % maximum rate of poisson inhibition train in Hz
rinh = 20;                    % the current firing rate. it is initialized to this value

firingrate1 = PRATE1;                                         % Firing rate in Hz of extranetwork excitatory poisson neurons
firingrate23 = PRATE23;

% Making connections

% layer 1
poissonconnections1 = zeros(numlay1,numexccon1,'single');
for i = 1:numlay1
    random = randperm(numexcsrc1,numexccon1);
    poissonconnections1(i,:) = random;
    clear random
end
% layer 2
poissonconnections2 = zeros(numlay2,numexccon2,'single');
for i = 1:numlay2
    random = randperm(numexcsrc23,numexccon2);
    poissonconnections2(i,:) = random;
    clear random
end
% layer 3
poissonconnections3 = zeros(numlay3,numexccon3,'single');
for i = 1:numlay3
    random = randperm(numexcsrc23,numexccon3);
    poissonconnections3(i,:) = random;
    clear random
end

% layer 1
inhconnections1 = zeros(numlay1,numinhcon1,'single');
for i = 1:numlay1
    random = randperm(numinhsrc,numinhcon1);
    inhconnections1(i,:) = random;
    clear random
end
% layer 2
inhconnections2 = zeros(numlay2,numinhcon2,'single');
for i = 1:numlay2
    random = randperm(numinhsrc,numinhcon2);
    inhconnections2(i,:) = random;
    clear random
end
% layer 3
inhconnections3 = zeros(numlay3,numinhcon3,'single');
for i = 1:numlay3
    random = randperm(numinhsrc,numinhcon3);
    inhconnections3(i,:) = random;
    clear random
end

PSM1 = -log(rand(numexcsrc1,1))/(firingrate1/1000);                    % extra network excitatory poisson spikes
PSM23 = -log(rand(numexcsrc23,1))/(firingrate23/1000);                    % extra network excitatory poisson spikes
IIP = -log(rand(numinhsrc,1))/(rinh/1000);                              % extra network inhibitory spikes for layer 1

placeholderextra1 = zeros(size(gexcextra1,1),size(gexcextra1,2),'single');
placeholderextra2 = zeros(size(gexcextra2,1),size(gexcextra2,2),'single');
placeholderextra3 = zeros(size(gexcextra3,1),size(gexcextra3,2),'single');
dgchange = zeros(numneuro,numneuro,'single');
dgextrachange1 = zeros(numlay1,numexccon1,'single');
dgextrachange2 = zeros(numlay2,numexccon2,'single');
dgextrachange3 = zeros(numlay3,numexccon3,'single');

%% Main loop

tic;
for q = 2:k
    
    t = dt*q;
    
    %Layer 1
    gexceff1 = sum(gexcextra1,2) + sum(gexcintra(1:numlay1,:),2);
    ginheff1 = sum(ginh1,2);
    for i = 1:numlay1
        Vinf = (Vrest + gexceff1(i)*Eexc + ginheff1(i)*Einh)/(1 + gexceff1(i) + ginheff1(i));
        taueff = taum/(1 + gexceff1(i) + ginheff1(i));
        V(i,q) = Vinf + (V(i,q-1)-Vinf)*exp(-dt/taueff);
        if V(i,q) > Vthresh
            spikes(i,q) = 1;
            gamma(q) = gamma(q) + (1/numneuro);
            V(i,q) = Vreset;
        end
    end
    % Layer 2
    gexceff2 =  sum(gexcextra2,2) + sum(gexcintra((numlay1+1):(numlay1+numlay2),:),2);
    ginheff2 = sum(ginh2,2);
    for i = 1:numlay2
        Vinf = (Vrest + gexceff2(i)*Eexc + ginheff2(i)*Einh)/(1 + gexceff2(i) + ginheff2(i));
        taueff = taum/(1 + gexceff2(i) + ginheff2(i));
        V((numlay1+i),q) = Vinf + (V((numlay1+i),q-1)-Vinf)*exp(-dt/taueff);
        if V((numlay1+i),q) > Vthresh
            spikes((numlay1+i),q) = 1;
            gamma(q) = gamma(q) + (1/numneuro);
            V((numlay1+i),q) = Vreset;
        end
    end
        % Layer 3
    gexceff3 =  sum(gexcextra3,2) + sum(gexcintra((numlay1+numlay2+1):(numlay1+numlay2+numlay3),:),2);
    ginheff3 = sum(ginh3,2);
    for i = 1:numlay3
        Vinf = (Vrest + gexceff3(i)*Eexc + ginheff3(i)*Einh)/(1 + gexceff3(i) + ginheff3(i));
        taueff = taum/(1 + gexceff3(i) + ginheff3(i));
        V((numlay1+numlay2+i),q) = Vinf + (V((numlay1+numlay2+i),q-1)-Vinf)*exp(-dt/taueff);
        if V((numlay1+numlay2+i),q) > Vthresh
            spikes((numlay1+numlay2+i),q) = 1;
            gamma(q) = gamma(q) + (1/numneuro);
            V((numlay1+numlay2+i),q) = Vreset;
        end
    end
    
    % Intranetwork weights
    M = M.*exp(-dt/taun);
    Pintra = Pintra.*exp(-dt/taup);
    dgchange = dgchange.*0;
    for i = 1:numneuro
        if spikes(i,q)==1
            M(i) = M(i)-An;
            Pintra(i) = Pintra(i)+Ap;
        end
    end
    for i =1:numneuro
        if spikes(i,q)==1
            for j =  1:numneuro
%                 dgchange(i,j) = dgchange(i,j) + prelearningrules_intra(dgexcintra(i,j), dgexcmax, mu, Pintra(j), type, layermap(i), layermap(j));                
                dgchange(i,j) = dgchange(i,j) + type(layermap(i), layermap(j))*((dgexcmax-dgexcintra(i,j))^mu).*Pintra(j) + (1 - type(layermap(i), layermap(j)))*(-(dgexcintra(i,j)^mu).*Pintra(j));                
%                 dgchange(j,i) = dgchange(j,i) + postlearningrules_intra(dgexcintra(j,i), dgexcmax, mu, M(j), type, layermap(j), layermap(i));                
                dgchange(j,i) = dgchange(j,i) + type(layermap(j), layermap(i))*(dgexcintra(j,i)^mu).*M(j) + (1 - type(layermap(j), layermap(i)))*(-((dgexcmax-dgexcintra(j,i))^mu).*M(j));
            end
        end
    end
    dgexcintra = dgexcintra + dgchange;                     % note the point of dgchange is to remove computational artifacts during dgexcintra  update
    dgexcintra(dgexcintra>dgexcmax) = dgexcmax;
    dgexcintra(dgexcintra<dgexcmin) = dgexcmin;
    dgexcintra = dgexcintra - diag(diag(dgexcintra));    
    % retain sparse connections
    dgexcintra = dgexcintra.*spse;
    
    gexcintra = gexcintra.*exp(-dt/tauexc);                 % This is done to speed up computation and  is undone in the for loop for appropraite neurons
    for i = 1:numneuro
        if spikes(i,q)==1
            gexcintra(:,i) = gexcintra(:,i)/exp(-dt/tauexc) + dgexcintra(:,i);
        end
    end
    
    % Extranetwork weights
    dgextrachange1 = dgextrachange1.*0;
    dgextrachange2 = dgextrachange2.*0;
    dgextrachange3 = dgextrachange3.*0;
    placeholderextra1 = placeholderextra1.*0;
    placeholderextra2 = placeholderextra2.*0;
    placeholderextra3 = placeholderextra3.*0;
    Pextra1 = Pextra1.*exp(-dt/taup);
    Pextra2 = Pextra2.*exp(-dt/taup);
    Pextra3 = Pextra3.*exp(-dt/taup);
    for j = 1:numexcsrc1
        if PSM1(j) < t                                                                                   % if extranetwork neuron j fires
            PSM1(j) = PSM1(j) - log(rand)/(firingrate1/1000);
            
            [sel1, sel2] = find(poissonconnections1 == j);
            if ~isempty(sel1)
                for h = 1:length(sel1)
                    placeholderextra1(sel1(h),sel2(h)) = placeholderextra1(sel1(h),sel2(h)) + 1;
                    Pextra1(sel1(h), sel2(h)) = Pextra1(sel1(h), sel2(h)) + Ap;
                    dgextrachange1(sel1(h),sel2(h)) = dgextrachange1(sel1(h),sel2(h)) + (dgexcextra1(sel1(h),sel2(h))^mu).*M(sel1(h));  % cSTDP
                    % dgextrachange(sel1(h),sel2(h)) = dgextrachange(sel1(h),sel2(h)) + postlearningrules_extra(dgexcextra(sel1(h),sel2(h)), dgexcmax, mu, M(sel1(h)), type, colormap(sel1(h)));
                end
            end
            clear sel1 sel2 h
        end
    end
    
    for j =1:numexcsrc23
        if PSM23(j) < t
            PSM23(j) = PSM23(j) - log(rand)/(firingrate23/1000);
            
            [sel1, sel2] = find(poissonconnections2 == j);
            if ~isempty(sel1)
                for h = 1:length(sel1)
                    placeholderextra2(sel1(h),sel2(h)) = placeholderextra2(sel1(h),sel2(h)) + 1;
                    Pextra2(sel1(h), sel2(h)) = Pextra2(sel1(h), sel2(h)) + Ap;
                    dgextrachange2(sel1(h),sel2(h)) = dgextrachange2(sel1(h),sel2(h)) + (dgexcextra2(sel1(h),sel2(h))^mu).*M(numlay1+sel1(h));  % cSTDP
                    % dgextrachange(sel1(h),sel2(h)) = dgextrachange(sel1(h),sel2(h)) + postlearningrules_extra(dgexcextra(sel1(h),sel2(h)), dgexcmax, mu, M(sel1(h)), type, colormap(sel1(h)));
                end
            end
            clear sel1 sel2 h
            
            [sel1, sel2] = find(poissonconnections3 == j);
            if ~isempty(sel1)
                for h = 1:length(sel1)
                    placeholderextra3(sel1(h),sel2(h)) = placeholderextra3(sel1(h),sel2(h)) + 1;
                    Pextra3(sel1(h), sel2(h)) = Pextra3(sel1(h), sel2(h)) + Ap;
                    dgextrachange3(sel1(h),sel2(h)) = dgextrachange3(sel1(h),sel2(h)) + (dgexcextra3(sel1(h),sel2(h))^mu).*M(numlay1+numlay2+sel1(h));  % cSTDP
                    % dgextrachange(sel1(h),sel2(h)) = dgextrachange(sel1(h),sel2(h)) + postlearningrules_extra(dgexcextra(sel1(h),sel2(h)), dgexcmax, mu, M(sel1(h)), type, colormap(sel1(h)));
                end
            end
            clear sel1 sel2 h
            
        end
    end
    
    for i =1:numlay1
        if spikes(i,q)==1
            for j = 1:numexccon1
                dgextrachange1(i,j) = dgextrachange1(i,j) + ((dgexcmax-dgexcextra1(i,j))^mu).*Pextra1(i,j);     % cSTDP
            end
        end
    end
    dgexcextra1 =  dgexcextra1 + dgextrachange1;
    dgexcextra1(dgexcextra1>dgexcmax) = dgexcmax;
    dgexcextra1(dgexcextra1<dgexcmin) = dgexcmin;    
    gexcextra1 = gexcextra1.*placeholderextra1 + gexcextra1.*(1-placeholderextra1).*exp(-dt/tauexc) + placeholderextra1.*dgexcextra1;

    
    for i =1:numlay2
        if spikes(numlay1+i,q)==1
            for j = 1:numexccon2
                dgextrachange2(i,j) = dgextrachange2(i,j) + ((dgexcmax-dgexcextra2(i,j))^mu).*Pextra2(i,j);     % cSTDP
            end
        end
    end
    dgexcextra2 =  dgexcextra2 + dgextrachange2;
    dgexcextra2(dgexcextra2>dgexcmax) = dgexcmax;
    dgexcextra2(dgexcextra2<dgexcmin) = dgexcmin;    
    gexcextra2 = gexcextra2.*placeholderextra2 + gexcextra2.*(1-placeholderextra2).*exp(-dt/tauexc) + placeholderextra2.*dgexcextra2;
    
    for i =1:numlay3
        if spikes(numlay1+numlay2+i,q)==1
            for j = 1:numexccon3
                dgextrachange3(i,j) = dgextrachange3(i,j) + ((dgexcmax-dgexcextra3(i,j))^mu).*Pextra3(i,j);     % cSTDP
            end
        end
    end
    dgexcextra3 =  dgexcextra3 + dgextrachange3;
    dgexcextra3(dgexcextra3>dgexcmax) = dgexcmax;
    dgexcextra3(dgexcextra3<dgexcmin) = dgexcmin;    
    gexcextra3 = gexcextra3.*placeholderextra3 + gexcextra3.*(1-placeholderextra3).*exp(-dt/tauexc) + placeholderextra3.*dgexcextra3;

    % Inhibitory Neurons
    %  layer 1
    rinh = rinh + (rmax - rmin)*gamma(q);
    rinh = rinh*exp(-dt/taur);
    if rinh>rmax
        rinh=rmax;
    end
    if rinh<rmin
        rinh=rmin;
    end
    
    ginh1 = ginh1.*exp(-dt/tauinh);                 %  this is done to speed up program and this calculation is undone in the following for loop for the appropriate terms
    ginh2 = ginh2.*exp(-dt/tauinh);
    ginh3 = ginh3.*exp(-dt/tauinh);
    for i = 1:numinhsrc
        if IIP(i) < t
            IIP(i) = IIP(i) -log(rand)/(rinh/1000);
            
            [sel1, sel2] = find(inhconnections1 == i);
            if ~isempty(sel1)
                for h = 1:length(sel1)
                    ginh1(sel1(h),sel2(h)) = ginh1(sel1(h),sel2(h))/exp(-dt/tauinh) + dginh;
                end
            end
            clear sel1 sel2 h
            
            [sel1, sel2] = find(inhconnections2 == i);
            if ~isempty(sel1)
                for h = 1:length(sel1)
                    ginh2(sel1(h),sel2(h)) = ginh2(sel1(h),sel2(h))/exp(-dt/tauinh) + dginh;
                end
            end
            clear sel1 sel2 h
            
            [sel1, sel2] = find(inhconnections3 == i);
            if ~isempty(sel1)
                for h = 1:length(sel1)
                    ginh3(sel1(h),sel2(h)) = ginh3(sel1(h),sel2(h))/exp(-dt/tauinh) + dginh;
                end
            end
            clear sel1 sel2 h
        end
    end
    
    if mod(q,1000) == 0
        readout = [num2str(q*dt/1000), ' sec'];
        disp(readout)
        toc;
    end
    
    if mod(q,memsave) == 0
        weights(:,:,q/memsave) = dgexcintra.*100;
    end
    if mod(q,memsave2) == 0
        weightsextra1(:,:,q/memsave2) = dgexcextra1;
        weightsextra2(:,:,q/memsave2) = dgexcextra2;
        weightsextra3(:,:,q/memsave2) = dgexcextra3;
    end
    
end


%% Analysis
clear dum
W = zeros(3,3,size(weights,3));
for k = 1:size(weights,3)
    dum = weights(:,:,k);
    W(1,1,k) = mean(mean(dum(1:numlay1, 1:numlay1)));
    W(1,2,k) = mean(mean(dum(1:numlay1, numlay1+1:numlay1+numlay2)));
    W(1,3,k) = mean(mean(dum(1:numlay1, numlay1+numlay2+1:numlay1+numlay2+numlay3)));
    W(2,1,k) = mean(mean(dum(numlay1+1:numlay1+numlay2, 1:numlay1)));
    W(2,2,k) = mean(mean(dum(numlay1+1:numlay1+numlay2, numlay1+1:numlay1+numlay2)));
    W(2,3,k) = mean(mean(dum(numlay1+1:numlay1+numlay2, numlay1+numlay2+1:numlay1+numlay2+numlay3)));
    W(3,1,k) = mean(mean(dum(numlay1+numlay2+1:numlay1+numlay2+numlay3, 1:numlay1)));
    W(3,2,k) = mean(mean(dum(numlay1+numlay2+1:numlay1+numlay2+numlay3, numlay1+1:numlay1+numlay2)));
    W(3,3,k) = mean(mean(dum(numlay1+numlay2+1:numlay1+numlay2+numlay3, numlay1+numlay2+1:numlay1+numlay2+numlay3)));
    clear dum
end

clearvars -except type duration numneuro simnum W gamma weights

save([num2str(type(1,1)),num2str(type(1,2)),num2str(type(1,3)),num2str(type(2,1)),num2str(type(2,2)),num2str(type(2,3)),num2str(type(3,1)),num2str(type(3,2)),num2str(type(3,3)),'_sim',num2str(simnum),'.mat'],'-v7.3');

end
