
%% Set directory and load all types of STDP configurations. 1 = cSTDP. 0 = rSTDP.
% alltypes is vectorized such that each row corresponds to a different
% configuration. elements of each row indicate STDP type between two layers
% of the model:
% [L4->L4, L2/3->L4, L5/6->L4, L4->L2/3, L2/3->L2/3, L5/6->L2/3, L4->L5/6, L2/3->L5/6, L5/6->L5/6]
addpath(genpath('..'))
alltypes_vec = get_alltypes;
%% Load average weights from the simulations of all 512
simnum_vec = (6:10);
allW_time_avg = zeros(size(alltypes_vec,1),length(simnum_vec),3,3);
for f1 = 1:size(alltypes_vec,1)
    for f2 = 1:length(simnum_vec)
        load([num2str(alltypes_vec(f1,:),'%1d'),'_sim',num2str(simnum_vec(f2)),'.mat']);
        allW_time_avg(f1,f2,:,:) = squeeze(mean(W(:,:,550:600),3)); %average over last 5 seconds of simulation time
        clearvars -except alltypes_vec simnum_vec f1 f2 allW_time_avg
    end
end; clear f1 f2
%% Calculate success of each configuration for each of 5 runs of the simulations
allsuccess = zeros(size(alltypes_vec,1),length(simnum_vec));
for f1 = 1:size(alltypes_vec,1)
    for f2 = 1:length(simnum_vec)
        allsuccess(f1,f2) = success(squeeze(allW_time_avg(f1,f2,:,:)));
    end
end
%% Caluclate avg success across 5 simulations of same STDP configuration
% Also caculate std and sort the configurations based on success
successavg = mean(allsuccess,2);
[successavg_sorted,ord]=sort(successavg);
ord = flip(ord); successavg_sorted = flip(successavg_sorted);
allW_sorted = allW_time_avg(ord,:,:,:);
allW_sorted_avg = squeeze(mean(allW_sorted,2));
alltypes_vec_sorted = alltypes_vec(ord,:);
allsuccess_sorted = allsuccess(ord,:);
allW_sorted_avg = reshape(permute(allW_sorted_avg,[1 3 2]),512,9);
successstd_sorted = zeros(size(successavg_sorted));
for f1 = 1:size(allsuccess_sorted,1)
    successstd_sorted(f1) = std(allsuccess_sorted(f1,:));
end
% While alltypes_vec_sorted, allW_sorted_avg,
% successavg_sorted, and successstd_sorted can be used to make Table S2,
% the standard deviations for the weights are not calculated here. This is 
% because the data used here did not save the individual weight values but
% rather just average weight matrices. Standard deviations for weights were
% calculated elsewhere (see weights_tables_S2.m) by compiling all
% individual weights from across the 5 simulations into one big weight
% histogram then computed std of the histogram. That dataset is 500 GB and
% is not provided here 
%% Figure S3a: Success curve for all 512 configurations with default parameters
gray_color = [142,128,124]./255;
blue_color = [52,139,203]./255;
red_color = [240,102,123]./255;
figure(1)
set(gcf,'Position',[1977 210 700 800])
errorbar((1:512),successavg_sorted,successstd_sorted,'.','MarkerSize',10,'LineWidth',2,'Color',gray_color,'MarkerFaceColor',red_color,'MarkerEdgeColor',red_color)
hold on
plot([1, 512], [0.5, 0.5],'--','LineWidth',3,'Color',blue_color)
set(gca,'FontSize',16)
xlabel('STDP Configuration')
xlim([0 513])
ylabel('Success')
ylim([0.1 0.8])
%% Figure 3d: 
hist_color = [72,72,74]./255;
figure(2)
set(gcf,'Position',[260   227   705   486])
[N,X] = hist(successavg_sorted,(0:0.02:1));
bar(X,N,'facecolor',hist_color,'edgecolor',hist_color)
set(gca,'FontSize',16)
ylabel('# of Configurations')
xlim([0 1])
xlabel('Success')
%%












