addpath(genpath('..'));
[alltypes, modulations, simnums] = create_modulation_parameters;
names = {};
k = 0;

for j = 1:size(alltypes,1)
    for l = 1:length(modulations)
        for i = 1:5
            k = k+1;
            names{k} = ['STDP_3layers_',num2str(alltypes(j,:),'%1d'),'_modulation=',num2str(modulations(l)),'_sim',num2str(i),'.mat'];
        end
    end
end
names = names';
clearvars -except home_path results_path analysis_pat alltypes modulations simnums names

%% Load desired variables from the files in "names"

between_layers = [2,3,4,6,7,8];
configurations = [2 0 1 0 0 0; 1 2 1 0 0 0; 1 0 2 0 0 0; 1 0 1 2 0 0; 1 0 1 0 2 0; 2 0 1 0 0 1; 1 2 1 0 0 1; 1 0 2 0 0 1; 1 0 1 2 0 1; 1 0 1 0 2 1; 1 0 1 0 0 2; 2 2 2 2 2 2];
configs = zeros(size(configurations,1),length(modulations),40); % will contain, for each configuration and modulation, the indices of alltypes_big which correspond to that configuration
counts = ones(size(configurations,1),length(modulations));

W_cell = {};
weights_cell = {};
type_cell = {};
success_cell = {};
alltypes_big = zeros(length(names),9);

for i = 1:length(names)
    load(names{i})
    W_cell{i} = W;
    weights_cell{i} = weights;
    type_cell{i} = type_all;
    success_cell{i} = success(squeeze(W(:,:,end)));
    alltypes_big(i,:) = reshape(type',[1,9]);
    ind1 = find(ismember(configurations,alltypes_big(i,between_layers),'rows'));
    ind2 = find(modulations==modulation);
    configs(ind1,ind2,counts(ind1,ind2)) = i;
    counts(ind1,ind2) = counts(ind1,ind2)+1;
end
clearvars -except home_path results_path analysis_pat alltypes modulations simnums names W_cell weights_cell type_cell success_cell alltypes_big configurations configs

%% Success modulation curve for each configuration
success_modulations_means = zeros(size(configurations,1),length(modulations));
success_modulations_errors = zeros(size(configurations,1),length(modulations));
success_vec = cell2mat(success_cell);

for ind1 = 1:size(configurations,1)
    for ind2 = 1:length(modulations)
        success_modulations_means(ind1,ind2) = mean(success_vec(squeeze(configs(ind1,ind2,:))));
        success_modulations_errors(ind1,ind2) = std(success_vec(squeeze(configs(ind1,ind2,:))));
    end
end
%% Fig S4: Success modulations plot for each configuration
best_16 = [1 0 1 0 0 0; 1 0 1 0 0 1];
color = [52,139,203]./255;
figure()
set(gcf,'Position',[1977 210 1500 800]);
for ind1 = 1:size(configs,1)
    subplot(4,3,ind1)
    errorbar(modulations,success_modulations_means(ind1,:),success_modulations_errors(ind1,:),'LineWidth',3,'col',color)
    hold on
    % plot a * where the best 16 simulation is:
    for j = 1:2
        if sum((configurations(ind1,:)-best_16(j,:))~=0)==1
            if max((configurations(ind1,:)-best_16(j,:)))==1
                plot(1,success_modulations_means(ind1,end),'*','MarkerSize',15,'col',color)
            end
            if max((configurations(ind1,:)-best_16(j,:)))==2
                plot(0,success_modulations_means(ind1,1),'*','MarkerSize',15,'col',color)
            end
        end
    end
    %
    xlim([0 1]); ylim([0.4 0.8])
    set(gca,'FontSize',16)
    title([num2str(configurations(ind1,:),'%1d')],'FontSize',20)
end

% The titles of the plots are 6 numbers representing between layer
% connections with 1=cSTDP, 0=rSTDP, 2=modulated STDP type
% [L2/3->L4, L5/6->L4, L4->L2/3, L5/6->L2/3, L4->L5/6, L2/3->L5/6];

%%















